package wood.keith.opentools.javad;

import java.io.File;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.PrimeTime;
import com.borland.primetime.help.HelpManager;
import com.borland.primetime.help.ZipHelpBook;
import com.borland.primetime.help.ZipHelpTopic;
import com.borland.primetime.ide.Browser;
import com.borland.primetime.node.Project;
import com.borland.primetime.util.VetoException;
import com.borland.primetime.vfs.Url;
import com.borland.primetime.wizard.BasicWizard;
import com.borland.primetime.wizard.Wizard;
import com.borland.primetime.wizard.WizardAction;
import com.borland.primetime.wizard.WizardHost;
import com.borland.primetime.wizard.WizardManager;
import com.borland.primetime.wizard.WizardPage;

/**
 * This class defines the "JavaD" wizard at the highest level.
 * It sets the dialog pages to be displayed and generates the
 * Wizard classes as defined by user input.
 *
 * @author   Keith Wood (kbwood@iprimus.com.au)
 * @version  1.0  16 March 2003
 */
public class JavaDWizard extends BasicWizard {

  public static final String TITLE = "JavaD Wizard";
  public static final String VERSION = "1.0";

  // Internal variables
  private JavaDWizardPage1 wizardPage1;

  /**
   * Register the "JavaD" tool.
   * Provides the needed OpenTools interface required to register the
   * WizardAction which defines and creates this wizard.
   *
   * @param  majorVersion  the major version of the current OpenTools API
   * @param  minorVersion  the minor version of the current OpenTools API
   */
  public static void initOpenTool(byte majorVersion, byte minorVersion) {
    if (majorVersion != PrimeTime.CURRENT_MAJOR_VERSION) {
      return;
    }
    WizardManager.registerWizardAction(WIZARD_JavaD);
    if (PrimeTime.isVerbose()) {
      System.out.println("Loaded JavaD wizard v" + VERSION);
      System.out.println("Written by Keith Wood (kbwood@iprimus.com.au)");
    }
  }

  /**
   * Definition of "JavaD" WizardAction.
   * This is an OpenTools registered action defining where the wizard will
   * appear in the IDE and a factory for creating each instance of this wizard.
   */
  public static final WizardAction WIZARD_JavaD = new WizardAction(
      "JavaD", '\0', "Generate a Pascal skeleton for using JNI on a Java class",
      null, new ImageIcon(ClassLoader.getSystemResource(
      "wood/keith/opentools/javad/JavaDWizard.gif")), true, null) {

    protected Wizard createWizard() {
      return new JavaDWizard();
    }
  };

  /**
   * Wizard startup.
   * "JavaD" wizard is to become visible,
   * create and order the wizard pages to be displayed.
   *
   * @param  host  the WizardHost that owns this wizard instance.
   * @return  the initial WizardPage to show.
   */
  public WizardPage invokeWizard(WizardHost host) {
    setWizardTitle(TITLE);
    Project project = host.getBrowser().getProjectView().getActiveProject();
    wizardPage1 = new JavaDWizardPage1(project);
    addWizardPage(wizardPage1);
    return super.invokeWizard(host);
  }

  /** Perform the wizard's task based on user input. */
  protected void finish() throws VetoException {
    Browser browser = wizardHost.getBrowser();
    JBProject project = (JBProject)browser.getProjectView().getActiveProject();
    try {
      String fileName = JavaD.getFileName(
        wizardPage1.getClassName(), wizardPage1.getDirectory());
      if (new File(fileName).exists() &&
          JOptionPane.showConfirmDialog(wizardHost.getBrowser(),
          "File " + fileName + "\nalready exists. Overwrite it?",
          TITLE, JOptionPane.YES_NO_OPTION) != JOptionPane.YES_OPTION) {
        throw new VetoException();
      }
      JavaD.generateDelphiWrapper(wizardPage1.getClassName(),
        wizardPage1.getDirectory(), new JBuilderClassLoader(), true);
      if (wizardPage1.isOpenFile()) {
    		// Open the generated file in the Content Pane
  			browser.setActiveNode(project.getNode(new Url(new File(fileName))), true);
      }
      else {
        // Or, inform of its presence
        JOptionPane.showMessageDialog(wizardHost.getBrowser(),
          "File " + fileName + " generated",
          TITLE, JOptionPane.INFORMATION_MESSAGE);
      }
    }
    catch (Exception e) {
      e.printStackTrace();
      JOptionPane.showMessageDialog(wizardHost.getBrowser(),
        "Error during JavaD:\n" +
        (e.getMessage() == null ? e.getClass().getName() : e.getMessage()),
        TITLE, JOptionPane.ERROR_MESSAGE);
      throw new VetoException();
    }
  }

  /**
   * Wizard termination.
   * The Wizard has been completed (finished or cancelled) and any resource
   * cleanup to help the garbage collector can occur here.
   */
  public void wizardCompleted() {
  }

  /**
   * Provide help for the wizard.
   *
   * @param  page  the current page in the wizard
   * @param  host  the wizard host
   */
  public void help(WizardPage page, WizardHost host) {
    HelpManager.showHelp(new ZipHelpTopic(new ZipHelpBook("JavaDWizardDoc.jar"),
      this.getClass().getName().replace('.', '/') + ".html"),
      page.getPageComponent(host));
  }
}